@@ -4,7 +4,7 @@ use strict;
 use DBIx::DBO2::Record '-isasubclass';
 
 use DBIx::DBO2::Fields (
-  { name => 'id', field_type => 'number', required => 1 },
+  { name => 'id', field_type => 'sequential', },
   { name => 'disc_id', field_type => 'number', required => 1 },
   { name => 'name', field_type => 'string' },
   { name => 'duration', field_type => 'number' },
